import React, { useImperativeHandle, useEffect, useRef, forwardRef, useState } from 'react';
import { SafeAreaView, findNodeHandle, DeviceEventEmitter, NativeEventEmitter, Platform, PixelRatio, StyleSheet } from 'react-native';
import { MapViewManager, CTPEventEmitter } from './MapViewManager';
import { centerMap, createFragment, drawNavigationPath, setFollowBlueDot, setMapVenue, setUserPosition, clearWayfindingPath, wayfindToWaypoint, wayfindBetweenWaypoints, styleUnitsForDestination, addMapIcon } from './SendCommand';
import Compass from './Compass';
const MapView = (props, parentRef) => {
  const nodeRef = useRef(null);
  const compassStyle = StyleSheet.create({
    compass: {
      height: 64,
      width: 64,
      borderRadius: 5,
      marginVertical: 40,
      // backgroundColor: '#61dafb',
      alignItems: 'center',
      justifyContent: 'center',
      zIndex: 1,
      position: 'absolute',
      right: '10%',
      top: '10%',
      flex: 1,
      resizeMode: 'contain'
    }
  });
  const [compassTransform, setCompassTransform] = useState({
    rotate: '0deg'
  });
  const [onNavigationPathGeneratedValue, setOnNavigationPathGeneratedValue] = useState(null);
  const [onDestinationSelectedValue, setOnDestinationSelectedValue] = useState(null);
  const [onUserPositionUpdatedValue, setOnUserPositionUpdatedValue] = useState(null);
  const [onSearchDestinationsGeneratedValue, setOnSearchDestinationsGeneratedValue] = useState(null);
  const [onMapTappedGenericValue, setOnMapTappedGenericValue] = useState(null);
  // const [onScreenRotated, setOnScreenRotated] =
  //   useState<SCREEN_ROTATED_EVENT | null>(null);

  const [onErrorValue, setOnErrorValue] = useState(null);
  const [onVenueLoadedValue, setOnVenueLoadedValue] = useState(null);
  const [onGeofencesChangedValue, setOnGeofencesChangedValue] = useState(null);
  useEffect(() => {
    if (Platform.OS == 'android') {
      createFragment(findNodeHandle(nodeRef.current));
    }
    setTimeout(function () {
      setMapVenue({
        viewId: findNodeHandle(nodeRef.current),
        credentials: props.credentials,
        mapOptions: props.mapOptions
      });
    }, 500);
  }, []);
  useEffect(() => {
    var _props$onMapMessage$o, _props$onMapMessage;
    if (onDestinationSelectedValue != null) (_props$onMapMessage$o = (_props$onMapMessage = props.onMapMessage).onDestinationSelected) === null || _props$onMapMessage$o === void 0 || _props$onMapMessage$o.call(_props$onMapMessage, onDestinationSelectedValue);
  }, [onDestinationSelectedValue]);
  useEffect(() => {
    if (onNavigationPathGeneratedValue != null) {
      var _props$onMapMessage$o2, _props$onMapMessage2;
      let pathId, instructions, start, end, path;
      ({
        pathId,
        instructions,
        start,
        end,
        path
      } = onNavigationPathGeneratedValue);
      (_props$onMapMessage$o2 = (_props$onMapMessage2 = props.onMapMessage).onNavigationPathGenerated) === null || _props$onMapMessage$o2 === void 0 || _props$onMapMessage$o2.call(_props$onMapMessage2, pathId, instructions, start, end, path);
    }
  }, [onNavigationPathGeneratedValue]);
  useEffect(() => {
    var _props$onMapMessage$o3, _props$onMapMessage3;
    if (onUserPositionUpdatedValue != null) (_props$onMapMessage$o3 = (_props$onMapMessage3 = props.onMapMessage).onUserPositionUpdated) === null || _props$onMapMessage$o3 === void 0 || _props$onMapMessage$o3.call(_props$onMapMessage3, onUserPositionUpdatedValue);
  }, [onUserPositionUpdatedValue]);
  useEffect(() => {
    var _props$onMapMessage$o4, _props$onMapMessage4;
    if (onSearchDestinationsGeneratedValue != null) (_props$onMapMessage$o4 = (_props$onMapMessage4 = props.onMapMessage).onSearchDestinationsGenerated) === null || _props$onMapMessage$o4 === void 0 || _props$onMapMessage$o4.call(_props$onMapMessage4, onSearchDestinationsGeneratedValue);
  }, [onSearchDestinationsGeneratedValue]);
  useEffect(() => {
    var _props$onMapMessage$o5, _props$onMapMessage5;
    if (onMapTappedGenericValue != null) (_props$onMapMessage$o5 = (_props$onMapMessage5 = props.onMapMessage).onMapTappedGeneric) === null || _props$onMapMessage$o5 === void 0 || _props$onMapMessage$o5.call(_props$onMapMessage5, onMapTappedGenericValue);
  }, [onMapTappedGenericValue]);
  useEffect(() => {
    var _props$onMapMessage$o6, _props$onMapMessage6;
    if (onErrorValue != null) (_props$onMapMessage$o6 = (_props$onMapMessage6 = props.onMapMessage).onError) === null || _props$onMapMessage$o6 === void 0 || _props$onMapMessage$o6.call(_props$onMapMessage6, onErrorValue);
  }, [onErrorValue]);
  useEffect(() => {
    var _props$onMapMessage$o7, _props$onMapMessage7;
    if (onVenueLoadedValue != null) (_props$onMapMessage$o7 = (_props$onMapMessage7 = props.onMapMessage).onVenueLoaded) === null || _props$onMapMessage$o7 === void 0 || _props$onMapMessage$o7.call(_props$onMapMessage7, onVenueLoadedValue);
  }, [onVenueLoadedValue]);
  // useEffect(() => {
  //   if (onScreenRotated != null)
  //     props.onMapMessage.onScreenRotated?.(onScreenRotated);
  // }, [onScreenRotated]);
  useEffect(() => {
    if (onGeofencesChangedValue != null) {
      var _props$onMapMessage$o8, _props$onMapMessage8;
      let entered, geofences;
      ({
        entered,
        geofences
      } = onGeofencesChangedValue);
      console.log("Geofences Changed: ", entered, geofences);
      (_props$onMapMessage$o8 = (_props$onMapMessage8 = props.onMapMessage).onGeofencesChanged) === null || _props$onMapMessage$o8 === void 0 || _props$onMapMessage$o8.call(_props$onMapMessage8, entered, geofences);
    }
  }, [onGeofencesChangedValue]);
  useImperativeHandle(parentRef, () => {
    const viewId = findNodeHandle(nodeRef.current);
    const listeners = {
      setMapVenue: (credentials, mapOptions) => {
        setTimeout(function () {
          setMapVenue({
            viewId,
            credentials,
            mapOptions
          });
        }, 500);
      },
      centerMapOnUser: function (orientation) {
        let zoom = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 5;
        let animationDuration = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1000;
        centerMap(viewId, orientation, zoom, animationDuration);
      },
      setFollowBlueDot: function (enabled) {
        let rotate = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
        let interval = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 500;
        setFollowBlueDot(viewId, enabled, rotate, interval);
      },
      setUserPosition: (lat, long, heading, confidence) => {
        setUserPosition(viewId, lat, long, heading, confidence, null);
      },
      setUserPositionXY: (x, y, heading, confidence) => {
        setUserPosition(viewId, x, y, heading, confidence, 'xy');
      },
      clearWayfindingPath: () => {
        clearWayfindingPath(viewId);
      },
      wayfindToWaypoint: waypointId => {
        wayfindToWaypoint(viewId, waypointId);
      },
      wayfindBetweenWaypoints: (originId, destinationId) => {
        wayfindBetweenWaypoints(viewId, originId, destinationId);
      },
      drawNavigationPath: (pathId, _ref) => {
        let {
          shouldFocus,
          style,
          originIcon,
          destinationIcon
        } = _ref;
        drawNavigationPath(viewId, pathId, shouldFocus ?? true, style ?? {}, originIcon, destinationIcon);
      },
      styleUnitsForDestination: (destinationId, style) => {
        styleUnitsForDestination(viewId, destinationId, style);
      },
      addMapIcon: (coordinates, coordType, icon) => {
        addMapIcon(viewId, coordinates, coordType, icon);
      }
    };
    return listeners;
  });
  useEffect(() => {
    var _eventEmitter, _eventEmitter2, _eventEmitter3, _eventEmitter4, _eventEmitter5, _eventEmitter6, _eventEmitter7, _eventEmitter8, _eventEmitter9;
    console.log('Initializing Event Emitter');
    let eventEmitter = null;
    if (Platform.OS == 'android') {
      eventEmitter = DeviceEventEmitter;
    } else if (Platform.OS == 'ios') {
      console.log(CTPEventEmitter);
      eventEmitter = new NativeEventEmitter(CTPEventEmitter);
    }
    (_eventEmitter = eventEmitter) === null || _eventEmitter === void 0 || _eventEmitter.addListener('DESTINATION_SELECTED', data => setOnDestinationSelectedValue(data));
    (_eventEmitter2 = eventEmitter) === null || _eventEmitter2 === void 0 || _eventEmitter2.addListener('NAVIGATION_PATH_GENERATED', data => setOnNavigationPathGeneratedValue(data));
    (_eventEmitter3 = eventEmitter) === null || _eventEmitter3 === void 0 || _eventEmitter3.addListener('USER_POSITION_UPDATED', userPosition => setOnUserPositionUpdatedValue(userPosition));
    (_eventEmitter4 = eventEmitter) === null || _eventEmitter4 === void 0 || _eventEmitter4.addListener('GENERATED_SEARCH_DESTINATIONS', destinations => setOnSearchDestinationsGeneratedValue(destinations));

    // eventEmitter?.addListener('WINDOW_LOADED', () =>
    //   props.onMapMessage.onWindowLoaded?.()
    // );
    (_eventEmitter5 = eventEmitter) === null || _eventEmitter5 === void 0 || _eventEmitter5.addListener('MAP_TAPPED_GENERIC', event => setOnMapTappedGenericValue(event));
    (_eventEmitter6 = eventEmitter) === null || _eventEmitter6 === void 0 || _eventEmitter6.addListener('SCREEN_ROTATED', event => {
      // setOnScreenRotated(event);
      const newTransform = {
        rotate: Math.round(event.degrees).toString() + 'deg'
      };
      setCompassTransform(newTransform);
    }
    // props.onMapMessage.onMapTappedGeneric?.(data)
    );

    (_eventEmitter7 = eventEmitter) === null || _eventEmitter7 === void 0 || _eventEmitter7.addListener('ERROR', msg => setOnErrorValue(msg));
    (_eventEmitter8 = eventEmitter) === null || _eventEmitter8 === void 0 || _eventEmitter8.addListener('VENUE_LOADED', venueId => setOnVenueLoadedValue(venueId));
    (_eventEmitter9 = eventEmitter) === null || _eventEmitter9 === void 0 || _eventEmitter9.addListener('GEOFENCES_CHANGED', data => setOnGeofencesChangedValue(data));
    return () => {
      var _eventEmitter10, _eventEmitter11, _eventEmitter12, _eventEmitter13, _eventEmitter14, _eventEmitter15, _eventEmitter16, _eventEmitter17, _eventEmitter18;
      console.log('Removing Event Emitters');
      (_eventEmitter10 = eventEmitter) === null || _eventEmitter10 === void 0 || _eventEmitter10.removeAllListeners('DESTINATION_SELECTED');
      (_eventEmitter11 = eventEmitter) === null || _eventEmitter11 === void 0 || _eventEmitter11.removeAllListeners('NAVIGATION_PATH_GENERATED');
      (_eventEmitter12 = eventEmitter) === null || _eventEmitter12 === void 0 || _eventEmitter12.removeAllListeners('USER_POSITION_UPDATED');
      (_eventEmitter13 = eventEmitter) === null || _eventEmitter13 === void 0 || _eventEmitter13.removeAllListeners('GENERATED_SEARCH_DESTINATIONS');
      (_eventEmitter14 = eventEmitter) === null || _eventEmitter14 === void 0 || _eventEmitter14.removeAllListeners('MAP_TAPPED_GENERIC');
      (_eventEmitter15 = eventEmitter) === null || _eventEmitter15 === void 0 || _eventEmitter15.removeAllListeners('ERROR');
      (_eventEmitter16 = eventEmitter) === null || _eventEmitter16 === void 0 || _eventEmitter16.removeAllListeners('VENUE_LOADED');
      (_eventEmitter17 = eventEmitter) === null || _eventEmitter17 === void 0 || _eventEmitter17.removeAllListeners('SCREEN_ROTATED');
      (_eventEmitter18 = eventEmitter) === null || _eventEmitter18 === void 0 || _eventEmitter18.removeAllListeners('GEOFENCES_CHANGED');
    };
  }, []);
  return /*#__PURE__*/React.createElement(SafeAreaView, {
    style: {
      flex: 1,
      width: '100%',
      height: '100%'
    }
  }, /*#__PURE__*/React.createElement(Compass, {
    style: [compassStyle.compass, {
      transform: [compassTransform]
    }]
  }), /*#__PURE__*/React.createElement(SafeAreaView, null), MapViewManager ? /*#__PURE__*/React.createElement(MapViewManager, {
    style: Platform.OS == 'android' ? {
      // converts dpi to px, provide desired height
      height: PixelRatio.getPixelSizeForLayoutSize(props.style.height),
      // converts dpi to px, provide desired width
      width: PixelRatio.getPixelSizeForLayoutSize(props.style.width)
    } : {
      height: props.style.height / 1,
      width: props.style.width / 1
      // converts dpi to px, provide desired height
      // height: PixelRatio.getPixelSizeForLayoutSize(props.style.height),
      // converts dpi to px, provide desired width
      // width: PixelRatio.getPixelSizeForLayoutSize(props.style.width),
    },

    ref: nodeRef
  }) : null);
};
const mapViewFwRef = /*#__PURE__*/forwardRef(MapView);
// mapViewFwRef.propTypes = {
//   eventEmitter: PropTypes.
// }
export default mapViewFwRef;
//# sourceMappingURL=MapView.js.map